﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;

namespace WebClientDemo
{
    internal class DownloadManager
    {
        private Stopwatch _downloadStopWatch = null;
        private int _currentDownloadTotalBytes = 0;
        private int _currentDownloadBytesSinceStopWatch = 0;
        private bool _cancelDownload = false;
        private BackgroundWorker _bgWorker = null;
        private static readonly int BufferSize = 32768;
        /*****************请让用户来设置这些信息*******************/
        private static string UserProxyAddress = "";
        private static string ProxyUserName = "";
        private static string ProxyUserPasswd = "";
        private static string DomainName = "";
        /********************************************************/

        public DownloadManager(BackgroundWorker bgWorker)
        {
            _bgWorker = bgWorker;
            _downloadStopWatch = new Stopwatch();
        }

        public bool DownloadFile(string url, string fileName)
        {
            bool isDownloadSuccessfully = false;
            string failureReason = string.Empty;
            try
            {
                Stopwatch sw = Stopwatch.StartNew();
                if (!this.WebClientDownloadInstallerFile(url, fileName, out failureReason))
                {
                    isDownloadSuccessfully = false;
                }
                sw.Stop();

                FileInfo fileInfo = new FileInfo(fileName);
                int _downloadSpeed = (int)((((double)fileInfo.Length) / sw.Elapsed.TotalMilliseconds) * 1000);
                isDownloadSuccessfully = true;
            }
            catch
            {
                isDownloadSuccessfully = false;
            }

            return isDownloadSuccessfully;
        }

        public void Cancel()
        {
            this._cancelDownload = true;
        }

        private bool WebClientDownloadInstallerFile(string url, string fileName, out string failureReason)
        {
            failureReason = string.Empty;
            if (url == null)
            {
                throw new ArgumentNullException("url");
            }

            bool isDownloadSuccessfully = false;
            Stream stream = null;
            FileStream fileStream = null;
            string tempFileName = Path.GetTempFileName();
            this._downloadStopWatch.Start();
            try
            {
                Uri installerUrl = new Uri(url);
                this._currentDownloadTotalBytes = 0;
                this._currentDownloadBytesSinceStopWatch = 0;
                using (MyWebClient myHttpWebClient = new MyWebClient())
                {
                    if (!string.IsNullOrEmpty(UserProxyAddress))
                    {
                        myHttpWebClient.CustomWebProxy = new WebProxy(UserProxyAddress);
                        if (ProxyUserName != null && ProxyUserPasswd != null)
                        {
                            myHttpWebClient.CustomWebProxy.Credentials = new NetworkCredential(ProxyUserName, ProxyUserPasswd, DomainName);
                        }
                    }

                    fileStream = new FileStream(tempFileName, FileMode.Create);
                    stream = myHttpWebClient.OpenRead(installerUrl);
                    double contentLength = DownloadManager.GetContentLength(myHttpWebClient);
                    byte[] buffer = new byte[BufferSize];
                    long downloadedLength = 0;
                    long currentTimeSpanDataLength = 0;          
                    int currentDataLength;
                    while ((currentDataLength = stream.Read(buffer, 0, BufferSize)) > 0 && !this._cancelDownload)
                    {
                        fileStream.Write(buffer, 0, currentDataLength);
                        downloadedLength += (long)currentDataLength;
                        currentTimeSpanDataLength += (long)currentDataLength;
                        int intDownloadSpeed = 0;
                        if (this._downloadStopWatch.ElapsedMilliseconds > 800)
                        {
                            /***********************************/
                            // 计算上一个时间段内的下载速度
                            double num5 = (double)currentTimeSpanDataLength / 1024.0;
                            double num6 = (double)this._downloadStopWatch.ElapsedMilliseconds / 1000.0;
                            double doubleDownloadSpeed = num5 / num6;
                            /***********************************/
                            intDownloadSpeed = (int)Math.Round(doubleDownloadSpeed, 0);

                            // 本次网速计算完成后重置时间计时器和数据计数器，开始下次的计算
                            this._downloadStopWatch.Reset();
                            this._downloadStopWatch.Start();
                            currentTimeSpanDataLength = 0;
                        }
             
                        double doubleDownloadPersent = 0.0;
                        if (contentLength > 0.0)
                        {
                            doubleDownloadPersent = (double)downloadedLength / contentLength;
                            if (doubleDownloadPersent > 1.0)
                            {
                                doubleDownloadPersent = 1.0;
                            }
                        }

                        int intDownloadPersent = (int)(doubleDownloadPersent * 100);
                        if (intDownloadSpeed != 0)
                        {
                            DownloadInfo info = new DownloadInfo() { Message = "xxx", Persent = intDownloadPersent, Speed = intDownloadSpeed };
                            DownloadingStatusChanged(info);
                        }
                    }

                    if (this._cancelDownload)
                    {
                        DownloadInfo info = new DownloadInfo() { Message = "已取消下载", Persent = 100, Speed = 0 };
                        DownloadingStatusChanged(info);
                    }
                    else if (currentDataLength >= 0)
                    {
                        // downlown correct
                        isDownloadSuccessfully = true;
                    }
                }
            }
            catch (Exception ex2)
            {
                failureReason = ex2.Message;
            }
            finally
            {
                this._downloadStopWatch.Stop();
                if (fileStream != null)
                {
                    fileStream.Close();
                }
                if (stream != null)
                {
                    stream.Close();
                }
            }
            if (isDownloadSuccessfully)
            {
                if (File.Exists(fileName))
                {
                    Util.DeleteFileIfExists(fileName);
                }
                DownloadInfo info = new DownloadInfo() { Message = "xxx", Persent = 100, Speed = 0 };
                DownloadingStatusChanged(info);
                File.Move(tempFileName, fileName);
            }
            return isDownloadSuccessfully;
        }

        private static double GetContentLength(MyWebClient webClient)
        {
            double result = 0.0;
            if (webClient.ResponseHeaders["Content-Length"] != null)
            {
                string s = webClient.ResponseHeaders["Content-Length"];
                if (!double.TryParse(s, out result))
                {
                    result = 0.0;
                }
            }
            return result;
        }

        private void DownloadingStatusChanged(DownloadInfo info)
        {
            if (_bgWorker != null && info != null)
            {
                _bgWorker.ReportProgress(0, info);
            }
        }
    }
}
